
__all__ = [
    "World",

    "Bullet",

    "SteamSystem",
    "SmokeSystem",
    "DirtSystem",

    "SteamEmitter",
    "SteamEmitter2",
    "CannonEmitter",
    "SmokeEmitter",
    "CrashEmitter",
    "ExplosionEmitter",
    ]

from pygext.gl.all import *
from pygext.lazy import Random
from pygext.math.vecalg import *

from steamer.config import Config

class Bullet(Entity):
    image = "gfx/cannonball.png"
    layer = "objects"

    def init(self, node, cfg=Config):
        self.set(
            x = node.realx,
            y = node.realy,
            scale = 0.7,
            color = (200,200,200),
            )

        self.node = node

        vx,vy = sincos(node.angle, cfg.cannon_velocity)
        self.move = Move(vx,vy).do(self)
        self.do(RealTickFunc(self.tick))

        act = Delay(0.1) + CallFunc(self.add_collnode, "bullet", 10)
        act.do()
        
    def tick(self, e, cfg=Config):
        if self.deleted:
            return
        self.move.add_velocity(0, cfg.bullet_gravity)
        h = World.land.get_h(int(self.x))
        if self.y > 600-h:
            CrashEmitter.direction = 0
            World.dirt_system.new_emitter(CrashEmitter, self)
            self.delete()


class World(object):
    land = None
    
    def init(self):
        self.steam_system = SteamSystem()
        self.dirt_system = DirtSystem()
        self.smoke_system = SmokeSystem()
        self.fire_system = FireSystem()
        self.font = GLFont(("sans-serif", 26))

class SteamMutator(object):
    
    def mutate(self, particles, delta):
        land = World.land
        g = -20 * delta
        real = director.ticker.realtick
        for p in particles:
            p.vy += g
            x = int(p.x)
            h = land.get_h(x)
            d = p.y - (580-h)
            if d > 10:
                p.vx = 0
            if d > 0 and p.vy > 0:                
                if real:
                    p.vy *= 0.85

class SteamSystem(BitmapParticleSystem):
    image = "gfx/steam.png"
    layer = "steam"
    mutators = [
        SteamMutator(),
        ]

class SmokeSystem(BitmapParticleSystem):
    image = "gfx/steam.png"
    layer = "dirt"
    mutators = [
        LinearForce(0,-20),
        ]

class FireSystem(BitmapParticleSystem):
    image = "gfx/fireparticle.png"
    layer = "dirt"
    mutators = [
        LinearForce(0,100),
        ]

class SteamEmitter(PointEmitter):
    align_to_node = True
    direction = 180
    angle = 20
    velocity = 150
    color = (255,255,255,Random(200,250))
    num_particles = 1
    rotation = Random(360)
    rotation_delta = Random(-100,100)
    scale = Random(0.3,0.4)
    scale_delta = Random(0.4,0.6)
    life = Random(0.5,2)
    fade_time = 1

class SteamEmitter2(PointEmitter):
    align_to_node = False
    direction = 0
    num_emits = 30
    delay = 0.03
    angle = 200
    velocity = Random(40,80)
    num_particles = 1
    rotation = Random(360)
    rotation_delta = Random(-100,100)
    scale = Random(0.1,0.3)
    scale_delta = Random(0.5,0.7)
    life = Random(0.5,2)
    fade_time = 3
    color = (255,255,255,200)
    

class CannonEmitter(PointEmitter):
    align_to_node = True    
    direction = 0
    angle = 240
    num_particles = 30
    num_emits = 1
    velocity = Random(20,100)
    scale = Random(0.2, 0.5)
    scale_delta = Random(0.3, 0.8)
    life = Random(0.5,1.5)
    fade_time = 1
    color = (100,100,100,255)

class SmokeEmitter(PointEmitter):
    align_to_node = False
    
    direction = 0
    angle = 20
    num_particles = 1
    delay = Random(0.01, 0.1)
    scale = Random(0.1, 0.15)
    scale_delta = Random(0.15, 0.25)
    velocity = Random(10,50)
    life = Random(2,4)
    fade_time = 1.5
    color = (120,120,120,Random(180,230))

class ExplosionEmitter(PointEmitter):
    align_to_node = False

    direction = 0
    angle = 360

    num_particles = 10
    num_emits = 1
    color = (255,255,255,Random(150,200))
    velocity = Random(50,150)
    scale = Random(0.5,1.2)
    scale_delta = -0.5
    life = Random(0.5,1.0)
    fade_time = 1
    

class DirtSystem(BitmapParticleSystem):
    image = "gfx/dirtparticle.png"
    layer = "dirt"
    mutators = [
        LinearForce(0,200),
        ]

class CrashEmitter(CircleEmitter):
    tangent = False
    radius = 20
    
    direction = 0
    angle = 160
    velocity = Random(10, 150)
    num_particles = 50
    num_emits = 1
    scale = Random(0.5,1.0)
    rotation = Random(360)
    rotation_delta = Random(-150,150)
    life = Random(1.5,3)
    fade_time = 0.1


World = World()
